# coding: utf-8

import representatives_votes.models as models
from representatives.models import Representative
from rest_framework import serializers


class VoteSerializer(serializers.ModelSerializer):
    """
    Vote serializer
    """
    proposal = serializers.CharField(
        source='proposal.fingerprint'
    )
    representative = serializers.CharField(
        source='representative.fingerprint',
        allow_null=True
    )

    class Meta:
        model = models.Vote
        fields = (
            'id',
            'proposal',
            'representative',
            'representative_name',
            'position'
        )

    def to_internal_value(self, data):
        data = super(VoteSerializer, self).to_internal_value(data)
        data['proposal'] = models.Proposal.objects.get(
            fingerprint=data['proposal']['fingerprint']
        )
        if data['representative']['fingerprint']:
            data['representative'] = Representative.objects.get(
                fingerprint=data['representative']['fingerprint']
            )
        else:
            data['representative'] = None

        return data


class ProposalSerializer(serializers.ModelSerializer):
    dossier = serializers.CharField(
        source='dossier.fingerprint'
    )

    dossier_title = serializers.CharField(
        source='dossier.title',
        read_only=True
    )

    dossier_reference = serializers.CharField(
        source='dossier.reference',
        read_only=True
    )

    class Meta:
        model = models.Proposal
        fields = (
            'id',
            'fingerprint',
            'dossier',
            'dossier_title',
            'dossier_reference',
            'title',
            'description',
            'reference',
            'datetime',
            'kind',
            'total_abstain',
            'total_against',
            'total_for',
            'url',
        )

    def to_internal_value(self, data):
        validated_data = super(ProposalSerializer, self).to_internal_value(
            data)
        validated_data['dossier'] = models.Dossier.objects.get(
            fingerprint=validated_data['dossier']['fingerprint']
        )
        validated_data['votes'] = data['votes']
        return validated_data

    def _create_votes(self, votes_data, proposal):
        for vote in votes_data:
            serializer = VoteSerializer(data=vote)
            if serializer.is_valid():
                serializer.save()
            else:
                raise Exception(serializer.errors)

    def create(self, validated_data):
        votes_data = validated_data.pop('votes')
        proposal = models.Proposal.objects.create(
            **validated_data
        )
        self._create_votes(votes_data, proposal)
        return proposal

    def update(self, instance, validated_data):
        validated_data.pop('votes')
        for attr, value in validated_data.iteritems():
            setattr(instance, attr, value)
        instance.save()
        return instance


class ProposalDetailSerializer(ProposalSerializer):
    """ Proposal serializer that includes votes """
    votes = VoteSerializer(many=True)

    class Meta(ProposalSerializer.Meta):
        fields = ProposalSerializer.Meta.fields + (
            'votes',
        )


class DossierSerializer(serializers.ModelSerializer):
    """ Base dossier serializer """
    class Meta:
        model = models.Dossier
        fields = (
            'id',
            'fingerprint',
            'title',
            'reference',
            'text',
            'link',
            'url',
        )


class DossierDetailSerializer(DossierSerializer):
    """
    Dossier serializer that includes proposals and votes.
    """
    proposals = ProposalDetailSerializer(many=True)

    class Meta(DossierSerializer.Meta):
        fields = DossierSerializer.Meta.fields + (
            'proposals',
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Dossier',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('fingerprint', models.CharField(unique=True, max_length=40)),
                ('title', models.CharField(max_length=1000)),
                ('reference', models.CharField(max_length=200)),
                ('text', models.TextField(default=b'', blank=True)),
                ('link', models.URLField()),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Proposal',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('fingerprint', models.CharField(unique=True, max_length=40)),
                ('title', models.CharField(max_length=1000)),
                ('description', models.TextField(default=b'', blank=True)),
                ('reference', models.CharField(max_length=200, null=True, blank=True)),
                ('datetime', models.DateTimeField()),
                ('kind', models.CharField(default=b'', max_length=200, blank=True)),
                ('total_abstain', models.IntegerField()),
                ('total_against', models.IntegerField()),
                ('total_for', models.IntegerField()),
                ('dossier', models.ForeignKey(related_name='proposals', to='representatives_votes.Dossier')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Vote',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('representative_name', models.CharField(max_length=200, null=True, blank=True)),
                ('representative_remote_id', models.CharField(max_length=200, null=True, blank=True)),
                ('position', models.CharField(max_length=10, choices=[(b'abstain', b'abstain'), (b'for', b'for'), (b'against', b'against')])),
                ('proposal', models.ForeignKey(related_name='votes', to='representatives_votes.Proposal')),
            ],
        ),
    ]



from django import template
from django.utils.safestring import mark_safe

register = template.Library()


@register.filter
def position_icon(position):
    if position == 'for':
        return mark_safe(
            '<i \
            aria-label="for" \
            class="fa fa-thumbs-up vote_positive" \
            title="for" \
            ></i>')
    elif position == 'against':
        return mark_safe(
            '<i \
            aria-label="against" \
            class="fa fa-thumbs-down vote_negative" \
            title="against" \
            ></i>')
    else:
        return mark_safe(
            '<i \
            aria-label="abstain" \
            class="fa fa-circle-o vote_abstain" \
            title="abstain" \
            ></i>')



from .models import (
    Dossier,
    Proposal,
    Vote
)

from rest_framework import (
    filters,
    viewsets,
)

from representatives.api import DefaultWebPagination

from representatives_votes.serializers import (
    DossierDetailSerializer,
    DossierSerializer,
    ProposalDetailSerializer,
    ProposalSerializer,
    VoteSerializer,
)


class DossierViewSet(viewsets.ReadOnlyModelViewSet):
    """
    API endpoint that allows dossiers to be viewed.
    """

    pagination_class = DefaultWebPagination
    queryset = Dossier.objects.all()
    serializer_class = DossierSerializer

    filter_backends = (
        filters.DjangoFilterBackend,
        filters.SearchFilter,
        filters.OrderingFilter
    )

    filter_fields = {
        'fingerprint': ['exact'],
        'title': ['exact', 'icontains'],
        'reference': ['exact', 'icontains'],
    }

    search_fields = (
        'title',
        'fingerprint',
        'reference',
        'text',
        'proposals__title'
    )

    ordering_fields = ('id', 'reference')

    def list(self, request):
        return super(DossierViewSet, self).list(request)

    def retrieve(self, request, pk=None):
        self.serializer_class = DossierDetailSerializer
        return super(DossierViewSet, self).retrieve(request, pk)


class ProposalViewSet(viewsets.ReadOnlyModelViewSet):
    """
    API endpoint that allows proposals to be viewed.
    """

    pagination_class = DefaultWebPagination
    queryset = Proposal.objects.select_related('dossier')
    serializer_class = ProposalSerializer

    filter_backends = (
        filters.DjangoFilterBackend,
        filters.SearchFilter,
        filters.OrderingFilter
    )

    filter_fields = {
        'fingerprint': ['exact'],
        'dossier__fingerprint': ['exact'],
        'title': ['exact', 'icontains'],
        'description': ['icontains'],
        'reference': ['exact', 'icontains'],
        'datetime': ['exact', 'gte', 'lte'],
        'kind': ['exact'],
    }

    search_fields = (
        'title',
        'fingerprint', 'reference',
        'dossier__fingerprint',
        'dossier__title',
        'dossier__reference'
    )

    ordering_fields = ('id', 'reference')

    def list(self, request):
        return super(ProposalViewSet, self).list(request)

    def retrieve(self, request, pk=None):
        self.serializer_class = ProposalDetailSerializer
        return super(ProposalViewSet, self).retrieve(request, pk)


class VoteViewSet(viewsets.ReadOnlyModelViewSet):
    """
    API endpoint that allows proposals to be viewed.
    """

    pagination_class = DefaultWebPagination
    queryset = Vote.objects.select_related('representative', 'proposal')
    serializer_class = VoteSerializer

    filter_backends = (
        filters.DjangoFilterBackend,
        filters.SearchFilter,
        filters.OrderingFilter
    )

    filter_fields = {
        'proposal__fingerprint': ['exact'],
        'position': ['exact'],
        'representative_name': ['exact', 'icontains'],
        'representative': ['exact']
    }

    def list(self, request):
        return super(VoteViewSet, self).list(request)

    def retrieve(self, request, pk=None):
        return super(VoteViewSet, self).retrieve(request, pk)



from django.conf.urls import include, url

from rest_framework import routers

from representatives_votes.api import (
    DossierViewSet,
    ProposalViewSet,
    VoteViewSet,
)

router = routers.DefaultRouter()
router.register(r'dossiers', DossierViewSet)
router.register(r'proposals', ProposalViewSet)
router.register(r'votes', VoteViewSet)

urlpatterns = [
    url(r'api/', include(router.urls)),
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.db.models import Count


def remove_duplicate(apps, schema_editor):
    Vote = apps.get_model('representatives_votes', 'Vote')
    duplicates = Vote.objects.values('proposal_id',
            'representative_id').annotate(Count('id')).filter(id__count__gt=1)

    for duplicate in duplicates:
        remove = Vote.objects.filter(
                proposal_id=duplicate['proposal_id'],
                representative_id=duplicate['representative_id'])

        for i in remove.values_list('pk')[1:]:
            Vote.objects.get(pk=i[0]).delete()


class Migration(migrations.Migration):

    dependencies = [
        ('representatives_votes', '0005_make_dossier_reference_unique'),
    ]

    operations = [
        migrations.RunPython(remove_duplicate)
    ]



# coding: utf-8
import logging
import sys

import ijson
import django
from django.apps import apps

from representatives_votes.models import Dossier

logger = logging.getLogger(__name__)

URL = 'http://parltrack.euwiki.org/dumps/ep_dossiers.json.xz'
LOCAL_PATH = 'ep_dossiers.json.xz'


def parse_dossier_data(data):
    """Parse data from parltarck dossier export (1 dossier) Update dossier
    if it existed before, this function goal is to import and update a
    dossier, not to import all parltrack data
    """
    changed = False
    ref = data['procedure']['reference']

    logger.debug('Processing dossier %s', ref)

    try:
        dossier = Dossier.objects.get(reference=ref)
    except Dossier.DoesNotExist:
        dossier = Dossier(reference=ref)
        logger.debug('Dossier did not exist')
        changed = True

    if dossier.title != data['procedure']['title']:
        logger.debug('Title changed from "%s" to "%s"', dossier.title,
                     data['procedure']['title'])
        dossier.title = data['procedure']['title']
        changed = True

    source = data['meta']['source'].replace('&l=en', '')
    if dossier.link != source:
        logger.debug('Source changed from "%s" to "%s"', dossier.link, source)
        dossier.link = source
        changed = True

    if changed:
        logger.info('Updated dossier %s', ref)
        dossier.save()


def main(stream=None):
    if not apps.ready:
        django.setup()

    for data in ijson.items(stream or sys.stdin, 'item'):
        parse_dossier_data(data)



from django.conf.urls import url

import views

urlpatterns = [
    url(
        r'^dossier/(?P<pk>\d+)/$',
        views.DossierDetail.as_view(),
        name='dossier-detail'
    ),
    url(
        r'dossier/$',
        views.DossierList.as_view(),
        name='dossier-list'
    ),
]



import os

BASE_DIR = os.path.dirname(os.path.abspath(__file__))

SECRET_KEY = 'notsecret'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'test.db'),
    }
}

INSTALLED_APPS = (
    'django.contrib.staticfiles',
    'rest_framework',
    'representatives',
    'representatives_votes',
)

DEBUG = True
LANGUAGE_CODE = 'en-us'
TIME_ZONE = 'UTC'
ROOT_URLCONF = 'representatives_votes.tests.urls'

STATIC_URL = '/static/'
USE_I18N = True
USE_L10N = True
USE_TZ = True

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'simple': {
            'format': '%(levelname)s[%(module)s]: %(message)s'
        },
    },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'simple'
        },
    },
    'loggers': {
        'django': {
            'handlers': ['console'],
            'level': 'INFO'
        },
        'representatives': {
            'handlers': ['console'],
            'level': 'DEBUG'
        },
        'representatives_votes': {
            'handlers': ['console'],
            'level': 'DEBUG'
        },
    },
}



from django import test

from responsediff.response import Response


class RepresentativeManagerTest(test.TestCase):
    fixtures = ['representatives_votes_test.json']

    def functional_test(self, queries, url):
        with self.assertNumQueries(queries):
            result = test.client.Client().get(url)
        Response.for_test(self).assertNoDiff(result)

    def test_dossiers(self):
        self.functional_test(1, '/api/dossiers/?format=json')

    def test_proposals(self):
        self.functional_test(1, '/api/proposals/?format=json')

    def test_votes(self):
        self.functional_test(1, '/api/votes/?format=json')
